// naumann@stce.rwth-aachen.de

#include "f.hpp"

#include "dco.hpp"

#include <vector>

template<typename T, typename PT>
void gradient(size_t ncp, size_t ncs, const PT &eps, T &x_v, std::vector<T> &p_v, const std::vector<std::vector<PT>> &dW, std::vector<T> &dxdp) {
  using DCO_T=typename dco::gt1s<T>::type;
  size_t n=p_v.size();
  DCO_T x; std::vector<DCO_T> p(n); dco::value(p)=p_v; 
  for (size_t i=0;i<n;i++) {
    x=x_v;
    dco::derivative(p[i])=1;
    f(ncp,ncs,eps,x,p,dW);
    dxdp[i]=dco::derivative(x);
    dco::derivative(p[i])=0;
  }
  x_v=dco::value(x);
}  

template<typename T, typename PT>
void hessian(size_t ncp, size_t ncs, const PT &eps, T &x_v, std::vector<T> &p_v, const std::vector<std::vector<PT>> &dW, std::vector<T> &dxdp_v, std::vector<std::vector<T>> &ddxdpp) {
  using DCO_T=typename dco::gt1s<T>::type;
  size_t n=p_v.size();
  DCO_T x; std::vector<DCO_T> p(n), dxdp(n); dco::value(p)=p_v; 
  for (size_t i=0;i<n;i++) {
    x=x_v;
    dco::derivative(p[i])=1;
    gradient(ncp,ncs,eps,x,p,dW,dxdp); 
    dxdp_v[i]=dco::value(dxdp[i]);
    for (size_t j=0;j<n;j++) ddxdpp[i][j]=dco::derivative(dxdp[j]);
    dco::derivative(p[i])=0;
  }
  x_v=dco::value(x);
}  

#include "h_main.hpp"
